/*
 *
 * (C) COPYRIGHT 2012-2017, 2019 ARM Limited. All rights reserved.
 *
 * This program is free software and is provided to you under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation, and any use by you of this program is subject to the terms
 * of such GNU licence.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can access it online at
 * http://www.gnu.org/licenses/gpl-2.0.html.
 *
 * SPDX-License-Identifier: GPL-2.0
 *
 */

#include <mali_kbase.h>

static int kbasep_procfs_memory_current_seq_show(struct seq_file *sfile, void *data)
{
	struct kbase_context *kctx = (struct kbase_context *)sfile->private;

	seq_printf(sfile, "%u\n", atomic_read(&(kctx->used_pages)));

	return 0;
}

/*
 *  File operations related to procfs entry for memory_*
 */
static int kbasep_mtk_memory_current_procfs_open(struct inode *in, struct file *file)
{
	return single_open(file, kbasep_procfs_memory_current_seq_show, PDE_DATA(in));
}

static const struct file_operations kbasep_mtk_memory_current_procfs_fops = {
	.owner = THIS_MODULE,
	.open = kbasep_mtk_memory_current_procfs_open,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
};

static int kbasep_procfs_memory_max_seq_show(struct seq_file *sfile, void *data)
{
	struct kbase_context *kctx = (struct kbase_context *)sfile->private;

	seq_printf(sfile, "%u\n", atomic_read(&(kctx->max_used_pages)));

	return 0;
}

static int kbasep_mtk_memory_max_procfs_open(struct inode *in, struct file *file)
{
	return single_open(file, kbasep_procfs_memory_max_seq_show, PDE_DATA(in));
}

static const struct file_operations kbasep_mtk_memory_max_procfs_fops = {
	.owner = THIS_MODULE,
	.open = kbasep_mtk_memory_max_procfs_open,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
};

void kbasep_mtk_procfs_kctx_init(struct kbase_context *kctx)
{
	//create files
	proc_create_data("memory_current", 0644, kctx->kctx_pentry,
		&kbasep_mtk_memory_current_procfs_fops, (void *)kctx);

	proc_create_data("memory_max", 0644, kctx->kctx_pentry,
		&kbasep_mtk_memory_max_procfs_fops, (void *)kctx);
}
