/*------------------------------------------------------------------------------
 * MediaTek Inc. (C) 2018. All rights reserved.
 *
 * Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 *----------------------------------------------------------------------------*/

#ifdef CONFIG_MALI_MIDGARD_DVFS

#include <linux/delay.h>
#include <mali_kbase.h>
#include "mali_kbase_platform_mstar.h"
#include "mali_kbase_platform_mstar_m7642_reg.h"

/* */
#define ADJUST_STEP         48 /* in MHz */

extern int mali_gpu_clock;

/* Helper functions */
static void update_frequency(u32 frequency)
{
    mali_gpu_clock = frequency;
    /* set GPU clock */
    SET_REG(gpu_pll, ctrl1, (frequency/GPU_CLOCK_STEP));
    udelay(1);
}

/* DVFS callbacks */
static bool dvfs_enable(struct kbase_device* kbdev)
{
    KBASE_DEBUG_ASSERT(NULL != kbdev);

    kbdev->dvfs.current_frequency = kbdev->dvfs.dvfs_max_freq;
    kbdev->dvfs.enabled = 1;

    return true;
}

static bool dvfs_disable(struct kbase_device* kbdev)
{
    KBASE_DEBUG_ASSERT(NULL != kbdev);

    kbdev->dvfs.current_frequency = kbdev->dvfs.dvfs_max_freq; /* restore to the max frequency */
    kbdev->dvfs.enabled = 0;
    update_frequency(kbdev->dvfs.current_frequency);

    return true;
}

static bool dvfs_clock_up(struct kbase_device* kbdev)
{
    u32 new_freq;

    KBASE_DEBUG_ASSERT(NULL != kbdev);

    if (kbdev->dvfs.current_frequency >= kbdev->dvfs.dvfs_max_freq)
    {
        return true;
    }

    new_freq = kbdev->dvfs.current_frequency + ADJUST_STEP;

    if (new_freq > kbdev->dvfs.dvfs_max_freq)
    {
        new_freq = kbdev->dvfs.dvfs_max_freq;
    }
    else if (new_freq < kbdev->dvfs.dvfs_min_freq)
    {
        new_freq = kbdev->dvfs.dvfs_min_freq;
    }

    kbdev->dvfs.current_frequency = new_freq;
    update_frequency(kbdev->dvfs.current_frequency);

    return true;
}

static bool dvfs_clock_down(struct kbase_device* kbdev)
{
    u32 new_freq;

    KBASE_DEBUG_ASSERT(NULL != kbdev);

    if (kbdev->dvfs.current_frequency <= kbdev->dvfs.dvfs_min_freq)
    {
        return true;
    }

    new_freq = (kbdev->dvfs.current_frequency >= ADJUST_STEP) ? (kbdev->dvfs.current_frequency - ADJUST_STEP) : 0;

    if (new_freq < kbdev->dvfs.dvfs_min_freq)
    {
        new_freq = kbdev->dvfs.dvfs_min_freq;
    }
    else if (new_freq > kbdev->dvfs.dvfs_max_freq)
    {
        new_freq = kbdev->dvfs.dvfs_max_freq;
    }

    kbdev->dvfs.current_frequency = new_freq;
    update_frequency(kbdev->dvfs.current_frequency);

    return true;
}

static bool dvfs_enable_boost(struct kbase_device* kbdev)
{
    if (!kbdev)
    {
        return false;
    }

    kbdev->dvfs.boost_enabled= 1;
    kbdev->dvfs.dvfs_max_freq = MALI_BOOST_FREQ;

    return true;
}

static bool dvfs_disable_boost(struct kbase_device* kbdev)
{
    if (!kbdev)
    {
        return false;
    }

    kbdev->dvfs.boost_enabled = 0;
    kbdev->dvfs.dvfs_max_freq = MALI_MAX_FREQ;

    return true;
}

/* Platform functions */
int dvfs_init(struct kbase_device* kbdev)
{
    KBASE_DEBUG_ASSERT(NULL != kbdev);

    memset(&kbdev->dvfs, 0, sizeof(kbdev->dvfs));

    kbdev->dvfs.dvfs_max_freq = MALI_MAX_FREQ;
    kbdev->dvfs.dvfs_min_freq = MALI_MIN_FREQ;

    kbdev->dvfs.enabled = 0;
    kbdev->dvfs.enable = dvfs_enable;
    kbdev->dvfs.disable = dvfs_disable;
    kbdev->dvfs.clock_up =dvfs_clock_up;
    kbdev->dvfs.clock_down = dvfs_clock_down;
    kbdev->dvfs.enable_boost = dvfs_enable_boost;
    kbdev->dvfs.disable_boost = dvfs_disable_boost;

    /* enable after init */
    dvfs_enable(kbdev);

    return 0;
}

void dvfs_term(struct kbase_device* kbdev)
{

}

#endif /* CONFIG_MALI_MIDGARD_DVFS */
