/*------------------------------------------------------------------------------
 * MediaTek Inc. (C) 2019. All rights reserved.
 *
 * Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEKBSOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 *----------------------------------------------------------------------------*/

#include <mali_kbase.h>
#include "mali_kbase_platform_mstar.h"
#include "mali_kbase_platform_mstar_mt5862_reg.h"

#include <linux/delay.h>
#include "mstar/mstar_chip.h"

#define GPU_CLOCK_STEP              12

#define FPGA_TEST_RWROR 0
#define FPGA_TEST_DETILE 0

#if FPGA_TEST_DETILE
	/* gpu bridge hardcode setting */
	#define GPU_Y_BASE    (0x40200000) // bus address
	#define GPU_Y_SIZE    (0x007E9000) // 3840x2160
	#define GPU_UV_BASE   (0x409E9000) // Y_BASE + Y_SIZE
	#define GPU_UV_SIZE   (0x003F4800) // 3840x2160/2
	#define GPU_VID_PITCH (0xF00)      // now, no need to roundup to power of 2

	/* disp_mfdec hardcode setting */
	#define MFC_L_BASE    (0x2220)     // luma base addr
	#define MFC_C_BASE    (0x2320)     // chroma base addr
	#define MFC_BLD_PITCH (0xF00)      // 3840
	/* disp_mfdec detile size setting */
	#define TILE_16x32 0
	#define TILE_32x32 1
	#define TILE_32x16 2
	#define TILE_16x2  3
	#define TILE_SIZE TILE_32x32 //modify this when switch to different tile size

	/* init disp_mfdec about gpu part */
	void init_mfdec_GPU(void)
	{
	#define FB_TYPE_L_BASE   0
	#define FB_TYPE_C_BASE   1
	#define FB_TYPE_PITCH    2
	#define FB_TYPE_PROP     3

		printk("init_mfdec_GPU()\n");

		/* SW reset */
		SET_REG(disp_mfdec0, mfdec_detile_rst, 1);
		SET_REG(disp_mfdec0, mfdec_detile_rst, 0);

		/* Set frame buffer index */
		SET_REG(disp_mfdec0, prg_fb_idx, 0);

		/* Set frame buffer base */
		SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_L_BASE);
		SET_REG(disp_mfdec0, fb_para_data_low, 0x0000);
		SET_REG(disp_mfdec0, fb_para_data_high, MFC_L_BASE);
		SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_C_BASE);
		SET_REG(disp_mfdec0, fb_para_data_low, 0x0000);
		SET_REG(disp_mfdec0, fb_para_data_high, MFC_C_BASE);

		/* Set frame buffer pitch */
		SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PITCH);
		SET_REG(disp_mfdec0, fb_para_data_low, ((MFC_BLD_PITCH+31)>>5)<<5);
		SET_REG(disp_mfdec0, fb_para_data_high, 0x0);

		/* Set frame buffer properties */
		SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PROP);
		SET_REG(disp_mfdec0, fb_para_data_low, TILE_SIZE);
		SET_REG(disp_mfdec0, fb_para_data_high, 0x0);
	}

	/* init mfdec bridge */
	void init_mfdec_bridge_GPU(void)
	{
		printk("init_mfdec_bridge_GPU()\n");

		/* 1. enable/disable mfdec */
		SET_REG(gpu, mfdec_off, 0); /* enable mfdec */

		/* 2. Set frame idx */
		SET_REG(gpu, mfdec_adr_idx, 0);

		/* 3. Set frame pitch */
		SET_REG(gpu, video_pitch, GPU_VID_PITCH);

		/* 4. Set start & end addr of luma & chroma (4K byte unit, bus address) */
		SET_REG(gpu, mfdec_y_start_low,   ((GPU_Y_BASE>>12)&0xFFFF));
		SET_REG(gpu, mfdec_y_start_high,  ((GPU_Y_BASE>>28)&0xFFFF));
		SET_REG(gpu, mfdec_y_end_low,     (((GPU_Y_BASE+GPU_Y_SIZE+0xFFF)>>12)&0xFFFF));
		SET_REG(gpu, mfdec_y_end_high,    (((GPU_Y_BASE+GPU_Y_SIZE+0xFFF)>>28)&0xFFFF));
		SET_REG(gpu, mfdec_uv_start_low,  ((GPU_UV_BASE>>12)&0xFFFF));
		SET_REG(gpu, mfdec_uv_start_high, ((GPU_UV_BASE>>28)&0xFFFF));
		SET_REG(gpu, mfdec_uv_end_low,    (((GPU_UV_BASE+GPU_UV_SIZE+0xFFF)>>12)&0xFFFF));
		SET_REG(gpu, mfdec_uv_end_high,   (((GPU_UV_BASE+GPU_UV_SIZE+0xFFF)>>28)&0xFFFF));

		/* 5. Update & clean */
		SET_REG(gpu, mfdec_adr_update,    0x1);
		SET_REG(gpu, mfdec_adr_update,    0x0);

		/* 6. Set valid vector */
		SET_REG(gpu, mfdec_valid,         0x1); /* use reg 0 */

		/* 7. Set tile mode vector for 16x32 tile mode if necessary */
		//SET_REG(gpu, mfdec_16b_mode,      0x1); /* set reg 0 to mfdec mode or non 16x32 tile mode */
	}
#endif

#if FPGA_TEST_RWROR
void setDisableRrorWror(void)
{
    //disable rror
    SET_REG(rror, ror_inorder, 0x1);
    //disable wror
    SET_REG(wror, ror_inorder, 0x1);

    return;
}
#endif

/* Platform functions */
void init_registers(void)
{
#define MIU_BASE_TO_MB(base)        ((base) >> 20)                   /* in MB */
#define DRAM_SIZE_REG_TO_MB(reg)    ((reg > 0) ? (1 << (reg)) : 0)   /* in MB */

    /* set GPU clock */

    SET_REG(gpu_pll, ctrl0, 0x0);
    SET_REG(gpu_pll, ctrl1, (MALI_MAX_FREQ/GPU_CLOCK_STEP));
    udelay(10);

#if FPGA_TEST_RWROR
    /* default is enable, disable for zebu test */
    setDisableRrorWror();
#endif

    /* set MIU addr for zebu*/
    SET_REG(gpu, miu0_start, 0x0200);
    SET_REG(gpu, miu0_end, 0x0800);
    SET_REG(gpu, miu1_start, 0xffff);
    SET_REG(gpu, miu1_end, 0xffff);

    printk("new miu 0 start 0x%x\n", (unsigned int) (GET_REG(gpu, miu0_start)));
    printk("new miu 0 end 0x%x\n", (unsigned int) (GET_REG(gpu, miu0_end)));

#if 0
    /* set MIU addr */
    /* for real chip */
    SET_REG(gpu, miu0_start, MIU_BASE_TO_MB(MSTAR_MIU0_BUS_BASE));
    SET_REG(gpu, miu0_end, MIU_BASE_TO_MB(MSTAR_MIU0_BUS_BASE) + DRAM_SIZE_REG_TO_MB(GET_REG(miu0, dram_size)));
    SET_REG(gpu, miu1_start, MIU_BASE_TO_MB(MSTAR_MIU1_BUS_BASE));
    SET_REG(gpu, miu1_end, MIU_BASE_TO_MB(MSTAR_MIU1_BUS_BASE) + DRAM_SIZE_REG_TO_MB(GET_REG(miu1, dram_size)));
#endif

    /* enable RIU */
#ifdef MSTAR_RIU_ENABLED
    SET_REG(gpu, riu, 1);
#endif

    /* set read request length 0 to 64 */
    SET_REG(gpu, rreq_rd_len0, 0x4);
    /* set read request length 0 to 48 */
    SET_REG(gpu, rreq_rd_len1, 0x1);

    /* enable dynamic clock gating for gpu bridge */
    SET_REG(gpu, dynamic_clk_gating_en, 1);

    /* enable GPU clock */
    SET_REG(clkgen1, ckg_gpu, 0);
    udelay(1);

    SET_REG(gpu, soft_reset, 0);    /* disable MALI soft reset */
    SET_REG(gpu, soft_reset, 1);    /* enable MALI soft reset */
    SET_REG(gpu, soft_reset, 0);    /* disable MALI soft reset */
    udelay(1);

    /* NOTICE!!!!! WROR and RROR trigger must be set after bridge register is init or reset!!!! */
    /* trigger WROR hw arb update */
    SET_REG(rror, miu_hw_arb_update, 0x0);
    SET_REG(rror, miu_hw_arb_update, 0x1);
    /* trigger RROR hw arb update */
    SET_REG(wror, miu_hw_arb_update, 0x0);
    SET_REG(wror, miu_hw_arb_update, 0x1);

    printk("rror inorder 0x%x\n", (unsigned int) (GET_REG(rror, ror_inorder)));
    printk("wror inorder 0x%x\n", (unsigned int) (GET_REG(wror, ror_inorder)));

#if FPGA_TEST_DETILE
    init_mfdec_GPU();
    init_mfdec_bridge_GPU();
#endif
}

void power_on(void)
{
    SET_REG(clkgen1, ckg_gpu, 0);   /* enable GPU clock */
    SET_REG(gpu, sram_sd_en, 0);    /* power on SRAM */

    /* PULL DOWN GPU RESET */
    SET_REG(gpu, soft_reset, 0);
    udelay(1);

    /* NOTICE!!!!! WROR and RROR trigger must be set after bridge register is init or reset!!!! */
    /* trigger WROR hw arb update */
    SET_REG(rror, miu_hw_arb_update, 0x0);
    SET_REG(rror, miu_hw_arb_update, 0x1);
    /* trigger RROR hw arb update */
    SET_REG(wror, miu_hw_arb_update, 0x0);
    SET_REG(wror, miu_hw_arb_update, 0x1);

    printk("rror inorder 0x%x\n", (unsigned int) (GET_REG(rror, ror_inorder)));
    printk("wror inorder 0x%x\n", (unsigned int) (GET_REG(wror, ror_inorder)));
    printk("power on\n");
}

void power_off(void)
{
    SET_REG(gpu, soft_reset, 1);    /* reset GPU */
    SET_REG(gpu, sram_sd_en, 1);    /* power off SRAM */
    SET_REG(clkgen1, ckg_gpu, 1);   /* disable GPU clock */
    udelay(1);
    printk("power off\n");
}

void soft_reset(void)
{
    /* reset GPU hardcore & external GPU bridge circuit */
    SET_REG(gpu, soft_reset, 1);
    udelay(1);
    SET_REG(gpu, soft_reset, 0);

    /* GPU secure sw reset will clear one-way register (0x5b[0], reg_gpu_dram_mask_en)
     * and let GPU use normal AID, which can only R/W normal MIU range.
     */
    SET_REG(gpu, secure_sw_reset, 1);
    udelay(1);
    SET_REG(gpu, secure_sw_reset, 0);
}

#ifdef MSTAR_DISABLE_SHADER_CORES
int get_num_disabled_cores(void)
{
    return 0;
}
#endif

void get_dram_length(u32* miu0_length, u32* miu1_length, u32* miu2_length)
{
#define DRAM_SIZE_REG_TO_MB(reg)    ((reg > 0) ? (1 << (reg)) : 0)      /* in MB */
#define DRAM_SIZE_REG_TO_B(reg)     (DRAM_SIZE_REG_TO_MB(reg) << 20)    /* in Byte */

	*miu0_length = DRAM_SIZE_REG_TO_B(GET_REG(miu0, dram_size));
#if defined(MSTAR_MIU1_BUS_BASE) && defined(ARM_MIU1_BASE_ADDR)
	*miu1_length = DRAM_SIZE_REG_TO_B(GET_REG(miu1, dram_size));
#else
	*miu1_length = 0;
#endif
	*miu2_length = 0;

	return;
}
