/*------------------------------------------------------------------------------
 * MediaTek Inc. (C) 2019. All rights reserved.
 *
 * Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 *----------------------------------------------------------------------------*/

#ifndef _MTK_DTV_GPU_BRIDGE_H_
#define _MTK_DTV_GPU_BRIDGE_H_
#include <linux/spinlock.h>

#define MAX_BRIDGE_SLOT_NUM 64
extern int mfdec_used_id;

typedef enum
{
	BRIDGE_STATUS_UNACTIVE = 0,
	BRIDGE_STATUS_INITED = 1,
	BRIDGE_STATUS_END = 2,
} bridge_status;

typedef struct bridge_device
{
	spinlock_t bridge_lock;
	u8 slots[MAX_BRIDGE_SLOT_NUM];
	int status;
} bridge_device;

void init_bridge_device(bridge_device *device)
{
	if (device == NULL)
	{
		printk(KERN_DEBUG "!!![Mali] failed to allocate bridge device!\n");
		return;
	}

	if (device->status == BRIDGE_STATUS_INITED)
		return;

	memset(device, 0, sizeof(bridge_device));
	spin_lock_init(&device->bridge_lock);
	device->status = BRIDGE_STATUS_INITED;
	mfdec_used_id = 0;

	printk("[Mali] %s device = %p\n", __func__, device);
}

int acquire_slot(bridge_device *device, int max)
{
	int i, curr_id = 0xFF;
	if (device == NULL)
	{
		printk("[Mali] %s device = %p\n", __func__, device);
		return curr_id;
	}

	if (device->status != BRIDGE_STATUS_INITED)
	{
		printk(KERN_DEBUG "!!![Mali]%s bridge device is not inited!\n", __func__);
		return curr_id;
	}

	spin_lock(&device->bridge_lock);
	/* traverse and find avaliable slot */
	for (i = 0; i < max; i++)
	{
		if (device->slots[i] == 0)
		{
			device->slots[i] = 1;
			curr_id = i;
			break;
		}
	}
	spin_unlock(&device->bridge_lock);

	return curr_id;
}

void release_slot(bridge_device *device, int id)
{
	if (device != NULL)
	{
		spin_lock(&device->bridge_lock);
		device->slots[id] = 0;
		spin_unlock(&device->bridge_lock);
	}
}

void close_bridge_device(bridge_device *device)
{
	if (device != NULL)
	{
		int i = 0;
		spin_lock(&device->bridge_lock);
		/* traverse and check slots */
		for (i = 0; i < MAX_BRIDGE_SLOT_NUM; i++)
		{
			if (device->slots[i] == 1)
			{
				printk(KERN_DEBUG "[Mali] Cannot close bridge because of occupied slot: %d!\n", i);
				spin_unlock(&device->bridge_lock);
				return;
			}
		}

		/* clear data */
		device->status = BRIDGE_STATUS_UNACTIVE;
		spin_unlock(&device->bridge_lock);
	}
}
#endif /* _MTK_DTV_GPU_BRIDGE_H_ */
