/*------------------------------------------------------------------------------
 * MediaTek Inc. (C) 2018. All rights reserved.
 *
 * Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 *----------------------------------------------------------------------------*/

#include <mali_kbase.h>
#include <mtk_dtv_gpu_bridge.h>
#include <linux/delay.h>

#ifndef _KBASE_PLATFORM_MSTAR_M7332_MFDEC_H_
#define _KBASE_PLATFORM_MSTAR_M7332_MFDEC_H_

#define MAX_MFDEC_SLOT_NUM 16

#define FB_TYPE_BLD_BASE 0
#define FB_TYPE_L_BASE   0
#define FB_TYPE_C_BASE   1
#define FB_TYPE_PITCH    2
#define FB_TYPE_PROP     3

bridge_device mfdec_bridge;

void init_mfdec_reg(u8 is_tiled_yuv)
{
    SET_REG(disp_mfdec0, mfdec_clk_gated, 0);
    SET_REG(disp_mfdec0, mfdec_reset, 1);
    /* XXX: Workaround for mfdec reset issue */
    udelay(1000);
    SET_REG(disp_mfdec0, mfdec_reset, 0);

    SET_REG(disp_mfdec0, tagram_inv, 1);
    SET_REG(disp_mfdec0, vd_iommu_tagram_inv, 1);

    /* Set flush timeout */
    SET_REG(disp_mfdec0, mfd_ofs_cache, 0);
    SET_REG(disp_mfdec0, mfd_l_cache_flush_timeout_lb, 0);
    SET_REG(disp_mfdec0, mfd_l_cache_flush_timeout_ub, 0);

    if (is_tiled_yuv == 0)
    {
        /* 0: mvop/dipr; 1: cpu/gpu */
        SET_REG(disp_mfdec0, mfdec_xpu_sel, 1);
    }

    /* 0: use detile-only engine; 1: use mfdec engine */
    SET_REG(disp_mfdec0, gpu_mfdec_map, 1);
    udelay(1);

    /* enable mfdec */
    SET_REG(gpu, mfdec_off, 0);
    udelay(1);

    /* Set reg_cpx_pf_en=0 */
    SET_REG(disp_mfdec0, mfd_cpx_pf_en, 0);
    udelay(1);
}

void init_mfdec(void)
{
    init_bridge_device(&mfdec_bridge);
    init_mfdec_reg(1);
}

int set_mfdec_bridge(u8 id, u64 base, u32 height, u32 pitch, u8 is_tiled_yuv)
{
    u32 size_y, size_uv;
    u64 base_uv;
    u16 valid_id = 0x0;
    if (id == 0xFF)
    {
        id = acquire_slot(&mfdec_bridge, MAX_MFDEC_SLOT_NUM);
        if (id == 0xFF)
            return -1;
        else if (id == 0x0)
        {
            /* Re-init mfdec reg when first slot is occupied */
            init_mfdec_reg(is_tiled_yuv);
        }
        mfdec_used_id += (1 << id);
    }

    SET_REG(gpu, mfdec_adr_idx, (id & 0xF));

    /* Set frame pitch (roundup to power of 2) */
    SET_REG(gpu, video_pitch, pitch);

    /* Bifrost MMU table is bus address */
    size_y = ((pitch * height + 0xFFF) >> 12) << 12;
    base_uv = base + size_y;
    /* size of YUV_420SP(NV12) format */
    size_uv = ((2 * (pitch * height / 4) + 0xFFF) >> 12) << 12;

    /* Set start & end addr of luma & chroma (4K byte unit, bus address) */
    SET_REG(gpu, mfdec_y_start_low,   ((base >> 12) & 0xFFFF));
    SET_REG(gpu, mfdec_y_start_high,  ((base >> 28) & 0xFFFF));
    SET_REG(gpu, mfdec_y_end_low,     (((base + size_y) >> 12) & 0xFFFF));
    SET_REG(gpu, mfdec_y_end_high,    (((base + size_y) >> 28) & 0xFFFF));
    SET_REG(gpu, mfdec_uv_start_low,  ((base_uv >> 12) & 0xFFFF));
    SET_REG(gpu, mfdec_uv_start_high, ((base_uv >> 28) & 0xFFFF));
    SET_REG(gpu, mfdec_uv_end_low,    (((base_uv + size_uv) >> 12) & 0xFFFF));
    SET_REG(gpu, mfdec_uv_end_high,   (((base_uv + size_uv) >> 28) & 0xFFFF));

    /* Update & clean */
    SET_REG(gpu, mfdec_adr_update,    0x1);
    SET_REG(gpu, mfdec_adr_update,    0x0);

    /* Set valid vector, do not clear any valid bit while GPU is working */
    valid_id = GET_REG(gpu, mfdec_valid);
    valid_id |= (0x1 << id);
    SET_REG_NO_CLEAR(gpu, mfdec_valid,         (valid_id & 0xFFFF));

    return id;
}

int set_mfdec_detile(u8 id, u64 video_ybase, u64 video_uvbase, u32 pitch, u32 tile_mode, u32 flags)
{
    if (id == 0xFF)
        return -1;

    /* Set frame buffer index */
    SET_REG(disp_mfdec0, prg_fb_idx, id);

    /* Set frame buffer luma base */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_L_BASE);
    SET_REG(disp_mfdec0, fb_para_data_low, video_ybase);
    SET_REG(disp_mfdec0, fb_para_data_high, (video_ybase >> 16));
    udelay(1);

    /* Set frame buffer chroma base */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_C_BASE);
    SET_REG(disp_mfdec0, fb_para_data_low, video_uvbase);
    SET_REG(disp_mfdec0, fb_para_data_high, (video_uvbase >> 16));
    udelay(1);

    /* Set frame buffer pitch */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PITCH);
    SET_REG(disp_mfdec0, fb_para_data_low, pitch);
    SET_REG(disp_mfdec0, fb_para_data_high, 0x0);
    udelay(1);

    /* Set frame buffer property */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PROP);
    {
        u8 offset = 0x0;
        u32 property = 0x0;

        /* For IOMMU */
        offset = (((video_uvbase) >> 32) << 2) & (0xf);
        property += offset << 8;
        offset = (((video_ybase) >> 32) << 2) & (0xf);
        property += offset << 4;
        property += tile_mode;
        SET_REG(disp_mfdec0, fb_para_data_low, property);

        property = 0x0;
        if (flags != 0)
        {
            property = property | (flags >> 16);
        }
        SET_REG(disp_mfdec0, fb_para_data_high, property);
    }

    return 0;
}

int set_mfdec_decode(u8 id, u64 video_ybase, u64 video_uvbase, u64 bld_base, u32 bld_pitch, u32 codec_type, u32 flags)
{
    if (id == 0xFF)
        return -1;

    if (0 == GET_REG(disp_mfdec0, mfdec_xpu_sel))
    {
        printk(KERN_DEBUG "!!![Mali] XPU_SEL = 0, Re-init GPU-MFDEC\n");
        init_mfdec_reg(0);
    }

    /* invalidate cache to insure the correctness */
    SET_REG(disp_mfdec0, tagram_inv, 1);

    /* Set frame buffer index */
    SET_REG(disp_mfdec0, prg_fb_idx, id);

    /* Set frame buffer blength base */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_BLD_BASE);
    SET_REG(disp_mfdec0, fb_para_data_low, bld_base);
    SET_REG(disp_mfdec0, fb_para_data_high, (bld_base >> 16));
    udelay(1);

    /* Set frame buffer pitch */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PITCH);
    SET_REG(disp_mfdec0, fb_para_data_low, bld_pitch);
    SET_REG(disp_mfdec0, fb_para_data_high, 0x0);
    udelay(1);

    /* Set frame buffer property */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PROP);
    {
        u8 offset = 0x0;
        u32 property = 0x0;

        /* For IOMMU */
        offset = (((bld_base) >> 32) << 2) & (0xf);
        property += offset << 12;
        offset = (((video_uvbase) >> 32) << 2) & (0xf);
        property += offset << 8;
        offset = (((video_ybase) >> 32) << 2) & (0xf);
        property += offset << 4;
        /* BIT[3]   : 0/1 hevc/vp9 frame buffer, only for compressed frame buffer */
        if (codec_type <= 1)
            property += codec_type << 3;
        /* BIT[2]   : 0/1 uncompressed/compressed frame buffer */
        /* BIT[1]   : must be 32*16 mode for decompressing */
        property += 0x6;
        SET_REG(disp_mfdec0, fb_para_data_low, property);

        property = 0x0;
        if (flags != 0)
        {
            property = property | (flags >> 16);
        }
        SET_REG(disp_mfdec0, fb_para_data_high, property);
    }

    return 0;
}

int reset_mfdec_slot(u8 id)
{
    if (id >= MAX_MFDEC_SLOT_NUM)
        return -1;

    /* Set invalid vector */
    CLEAR_REG_VAL(gpu, mfdec_valid, (0x1 << id));

    release_slot(&mfdec_bridge, id);
    mfdec_used_id -= (1 << id);

    return 0;
}
#endif /* _KBASE_PLATFORM_MSTAR_M7332_MFDEC_H_ */
